
import os
import sys
import hashlib

from petcmd import Commander

from petdb.service.api import DEFAULT_PORT
from petdb.service.server import run

commander = Commander()

SERVICE_NAME = "petdb.database.{port}.service"
SERVICE_PATH = os.path.join("/etc/systemd/system", SERVICE_NAME)

template = f"""
[Unit]
Description=PetDB Service

[Service]
User={os.environ.get("USER", "root")}
Environment="LD_LIBRARY_PATH=/usr/local/lib"
Environment="PYTHONUNBUFFERED=1"
WorkingDirectory=/var/lib/petdb
ExecStart={sys.executable} -u -m petdb.service run "{{password_hash}}" -P {{port}}
Restart=always

[Install]
WantedBy=multi-user.target
""".strip()

def check_current_configuration(service_path: str) -> bool:
	try:
		with open(service_path, "r") as f:
			executable = f.read().split("ExecStart=")[1].split(" ")[0]
		if executable != sys.executable:
			print(f"PetDB service have already been configured with another python executable: {executable}")
			if input(f"Want to replace it with the current one: {sys.executable} (y/n)? ").lower() != "y":
				return False
	except Exception:
		pass
	return True

@commander.command("init", "reinit")
def init_service(password: str, port: int = DEFAULT_PORT):
	service_name = SERVICE_NAME.format(port=port)
	service_path = SERVICE_PATH.format(port=port)
	if os.path.exists(service_path):
		if not check_current_configuration(service_path):
			return
		os.system(f"sudo systemctl stop {service_name}")
		os.system(f"sudo systemctl disable {service_name}")
		os.remove(service_path)
		os.system(f"sudo systemctl daemon-reload")
	with open(service_path, "w") as f:
		f.write(template.format(password_hash=hashlib.sha256(password.encode("utf-8")).hexdigest(), port=port))
	os.system(f"sudo systemctl daemon-reload")
	os.system(f"sudo systemctl enable {service_name}")
	os.system(f"sudo systemctl start {service_name}")

@commander.command("run")
def run_service(password_hash: str, port: int):
	run(password_hash=password_hash, port=port)

if __name__ == "__main__":
	commander.process()




import threading
import collections

class QLock:

	def __init__(self):
		self.lock = threading.Lock()
		self.waiters = collections.deque()
		self.count = 0

	def acquire(self):
		self.lock.acquire()
		if self.count:
			new_lock = threading.Lock()
			new_lock.acquire()
			self.waiters.append(new_lock)
			self.lock.release()
			new_lock.acquire()
			self.lock.acquire()
		self.count += 1
		self.lock.release()

	def release(self):
		with self.lock:
			if not self.count:
				raise ValueError("lock not acquired")
			self.count -= 1
			if self.waiters:
				self.waiters.popleft().release()

	def locked(self):
		return self.count > 0

	def __enter__(self):
		self.acquire()

	def __exit__(self, exc_type, exc_val, exc_tb):
		self.release()




from petdb.pdb import PetDB
from petdb.pcollection import PetCollection, PetMutable, PetArray
from petdb.putils import NonExistent, NON_EXISTENT
from petdb.pexceptions import QueryException

__all__ = ["PetDB", "PetCollection", "PetArray", "NonExistent", "QueryException", "PetMutable", "NON_EXISTENT"]



import os
import sys
import time
import hashlib
import threading

import uvicorn
from fastapi import FastAPI, Request, Response, status, Body

from petdb import PetDB, PetCollection, PetArray
from petdb.service.api import DEFAULT_PORT
from petdb.service.qlock import QLock

if sys.platform != "linux":
	raise Exception("PetDB.service supports only Linux system")

STORAGE_PATH = "/var/lib/petdb"
if not os.path.exists(STORAGE_PATH):
	os.makedirs(STORAGE_PATH, exist_ok=True)

db = PetDB.get(STORAGE_PATH)
app = FastAPI()
lock = QLock()

@app.post("/collections")
def get_collections():
	with lock:
		return db.collections()

@app.post("/drop")
def drop_collections():
	with lock:
		db.drop()

@app.post("/drop/{name}")
def drop_collection(name: str):
	with lock:
		db.drop_collection(name)

@app.post("/mutate/{name}")
def mutate(name: str, mutations: list[dict] = Body(embed=True)):
	with lock:
		array = db.collection(name)
		for mutation in mutations:
			array: PetArray = array.__getattribute__(mutation["type"])(*mutation["args"])
		return array.list()

@app.post("/insert/{name}")
def insert(name: str, doc: dict = Body(embed=True)):
	with lock:
		return db.collection(name).insert(doc)

@app.post("/insert_many/{name}")
def insert_many(name: str, docs: list[dict] = Body(embed=True)):
	with lock:
		return db.collection(name).insert_many(docs)

@app.post("/update_one/{name}")
def update_one(name: str, update: dict = Body(embed=True), query: dict = Body(embed=True)):
	with lock:
		return db.collection(name).update_one(update, query)

@app.post("/update/{name}")
def update(name: str, update: dict = Body(embed=True), query: dict = Body(embed=True)):
	with lock:
		return db.collection(name).update(update, query)

@app.post("/remove/{name}")
def remove(name: str, query: dict = Body(embed=True)):
	with lock:
		return db.collection(name).remove(query)

@app.post("/clear/{name}")
def clear(name: str):
	with lock:
		return db.collection(name).clear()

def cache_monitor():
	while True:
		print("start cache checking...")
		now = int(time.time())
		with lock:
			instances = PetCollection.instances()
			for path in list(instances.keys()):
				print(f"check {instances[path]["instance"].name}...")
				if now - instances[path]["created"] > 3 * 24 * 3600:
					print(f"clear {instances[path]["instance"].name}")
					del instances[path]
		time.sleep(24 * 3600)

def run(port: int = DEFAULT_PORT, password_hash: str = ""):

	@app.middleware("http")
	async def authentication(request: Request, call_next):
		body = await request.json()
		if hashlib.sha256(body["password"].encode("utf-8")).hexdigest() == password_hash:
			return await call_next(request)
		return Response(status_code=status.HTTP_401_UNAUTHORIZED)

	threading.Thread(target=cache_monitor).start()

	uvicorn.run(app, host="127.0.0.1", port=port)
